@@ -1,12 +1,13 @@
 from __future__ import annotations
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
-from typing import Dict, List, Callable, Optional, Any, TYPE_CHECKING
+from typing import Dict, List, Callable, Optional, Any, TYPE_CHECKING, TypeVar
 import typing
 
 # Used for cyclic dependencies in type hints
 if TYPE_CHECKING:
-    from xdsl.parser import Parser, Printer
+    from xdsl.parser import Parser
+    from xdsl.printer import Printer
 
 
 @dataclass
@@ -30,7 +31,7 @@ class MLContext:
                 f"Attribute {attr.name} has already been registered")
         self._registeredAttrs[attr.name] = attr
 
-    def get_op(self, name: str) -> type:
+    def get_op(self, name: str) -> typing.Type[Operation]:
         if name not in self._registeredOps:
             raise Exception(f"Operation {name} is not registered")
         return self._registeredOps[name]
@@ -117,6 +118,9 @@ class ParametrizedAttribute(Attribute):
         ...
 
 
+T = TypeVar('T', bound='Operation')
+
+
 @dataclass
 class Operation(ABC):
     """A generic operation. Operation definitions inherit this class."""
@@ -149,12 +153,12 @@ class Operation(ABC):
         assert (isinstance(self.name, str))
 
     @staticmethod
-    def with_result_types(
-            op: Any,
-            operands: List[SSAValue],
-            result_types: List[Attribute],
-            attributes: Optional[Dict[str, Attribute]] = None,
-            successors: Optional[List[Block]] = None) -> Operation:
+    def with_result_types(op: Any,
+                          operands: List[SSAValue],
+                          result_types: List[Attribute],
+                          attributes: Optional[Dict[str, Attribute]] = None,
+                          successors: Optional[List[Block]] = None,
+                          regions: Optional[List[Region]] = None) -> Operation:
         operation = op()
         operation.operands = operands
         operation.results = [
@@ -165,8 +169,21 @@ class Operation(ABC):
             operation.attributes = attributes
         if successors is not None:
             operation.successors = successors
+        if regions is not None:
+            for region in regions:
+                operation.add_region(region)
         return operation
 
+    @classmethod
+    def create(cls: typing.Type[T],
+               operands: List[SSAValue],
+               result_types: List[Attribute],
+               attributes: Optional[Dict[str, Attribute]] = None,
+               successors: Optional[List[Block]] = None,
+               regions: Optional[List[Region]] = None) -> T:
+        return Operation.with_result_types(cls, operands, result_types,
+                                           attributes, successors, regions)
+
     def add_region(self, region: Region) -> None:
         self.regions.append(region)
         region.parent = self
